<?php


namespace SwaggerBake\Test\TestCase\Lib\Annotations;


use Cake\Routing\Route\DashedRoute;
use Cake\Routing\Router;
use Cake\Routing\RouteBuilder;
use Cake\TestSuite\TestCase;
use SwaggerBake\Lib\AnnotationLoader;
use SwaggerBake\Lib\EntityScanner;
use SwaggerBake\Lib\RouteScanner;
use SwaggerBake\Lib\Configuration;
use SwaggerBake\Lib\Swagger;

class SwagPathTest extends TestCase
{
    /** @var string[]  */
    public $fixtures = [
        'plugin.SwaggerBake.Employees',
        'plugin.SwaggerBake.EmployeeTitles',
    ];

    /** @var Router  */
    private $router;

    /** @var array  */
    private $config;

    public function setUp(): void
    {
        parent::setUp(); // TODO: Change the autogenerated stub
        $router = new Router();
        $router::scope('/api', function (RouteBuilder $builder) {
            $builder->setExtensions(['json']);
            $builder->resources('Employees', function (RouteBuilder $routes) {
                $routes->resources('EmployeeTitles');
            });
        });
        $this->router = $router;

        $this->config = [
            'prefix' => '/api',
            'yml' => '/config/swagger-bare-bones.yml',
            'json' => '/webroot/swagger.json',
            'webPath' => '/swagger.json',
            'hotReload' => false,
            'exceptionSchema' => 'Exception',
            'requestAccepts' => ['application/x-www-form-urlencoded'],
            'responseContentTypes' => ['application/json'],
            'namespaces' => [
                'controllers' => ['\SwaggerBakeTest\App\\'],
                'entities' => ['\SwaggerBakeTest\App\\'],
                'tables' => ['\SwaggerBakeTest\App\\'],
            ]
        ];

        AnnotationLoader::load();
    }

    public function testPath()
    {
        $config = new Configuration($this->config, SWAGGER_BAKE_TEST_APP);

        $cakeRoute = new RouteScanner($this->router, $config);

        $swagger = new Swagger(new EntityScanner($cakeRoute, $config));

        $arr = json_decode($swagger->toString(), true);
        $employees = $arr['paths']['/employees'];

        $this->assertEquals('summary here', $employees['summary']);
        $this->assertEquals('description here', $employees['description']);
    }

    public function testPathInvisible()
    {
        $config = new Configuration($this->config, SWAGGER_BAKE_TEST_APP);

        $cakeRoute = new RouteScanner($this->router, $config);

        $swagger = new Swagger(new EntityScanner($cakeRoute, $config));

        $arr = json_decode($swagger->toString(), true);

        $this->assertArrayNotHasKey('/employee-titles', $arr['paths']);
    }
}